from visual import *
from visual.controls import *
import math as m

print """
Waveplates:
- Rotate the waveplate with the slider
- Use the keyboard arrow keys
  to move the screen left/right/up/down
- change the type of waveplate in the source code
"""

#-----------Change these numbers to reflect the experimental Setup-----------------------------

deltan=0.25                                # Birefringence of Xtal: .25 means 1/4 Wave Plate
AngleOfWP=45	                            # Orientation of Waveplate


#----------------------------------------------------------------------------------------------

class EMwave:
    def __init__(self, StartPos=vector(0,0,0), PolDir=vector(0,1,0), Amplitude = 1.0,
                 K_Vector = vector(0,0,1), length = 20.0, phase = 0.0, color=(0,1,0)):
        nVecNorm=20
        self.PolDir=PolDir
        self.K_Vector=K_Vector
        self.Amplitude=Amplitude
        self.dVecPos=norm(K_Vector)*2*pi/nVecNorm	    #The distance between arrows assumes a wavevector of magnitude 1
        self.length=length	                            #This allows to "add" arrows from different waves, even with non equal wavelengths
        self.nVec=int(floor(self.length/mag(self.dVecPos)))
        self.StartPos = vector(StartPos)
        self.phase = phase
        self.PolDir=PolDir
        self.Amplitude=Amplitude
        #self.Elength=1
        self.Evecs = []
        for i in range(self.nVec):
                    arrowpos = self.StartPos+i*self.dVecPos
                    self.Evecs.append( arrow( pos=arrowpos, axis=self.PolDir, 
        			color = color, shaftwidth = 0.1 , fixedwidth = 1) )
        
    def CalcField(self,omega,t):
        for i in range(self.nVec):
            RelposInWave=i*self.dVecPos
            Elength=self.Amplitude*sin( dot(self.K_Vector,RelposInWave)-omega*t-self.phase)
            self.Evecs[i].axis = self.PolDir
            self.Evecs[i].length=Elength

def setphase(obj): # called on slider drag events
    if obj is s1:
        EInput2.PhiStart = obj.value
    else:
        EInput2.Amplitude = (1-s2.value)*EInput1.Amplitude



#----------------------------------------------------------------------------------------------
# Scene Settings

scene.width = 800
scene.height = 800
scene.center = (0,0,0)
scene.forward = (0,-0.3,-1)
scene.background = (0,0,0)
scene.x=100
scene.y=0
scene.select()
scene.autoscale=0
scene.title = "Waveplate"
scene.visible=1

#----------------------------------------------------------------------------------------------
# "Constants"
Llength=60	                            # Total Length of setup
Kdirection=norm(vector(1,0,0))              # Direction of Propagation
Startpositrue=(-0.5)*Kdirection*Llength     # Center Wave around Zero
Xtallength=0.7*Llength	                    # Length of Xtal
NWavelengths=Xtallength/2/pi	            # Number of Wavelengths in Xtal
deltan=1.0*deltan                           # floating point
dn=deltan/NWavelengths	                    # Birefringence of Xtal: .25 means 1/4 Wave Plate
AngleOfWP=1.0*AngleOfWP                     # must be floatingpoint
AngleOfWP=AngleOfWP/180*pi
Xtalup=(0,cos(AngleOfWP), sin(AngleOfWP))  # Orientation of the Xtal: First component must be "0"!!
#----------------------------------------------------------------------------------------------


#----------------------------------------------------------------------------------------------

#Xtal
Xtal=box(pos=(0,0,0), axis=Kdirection, length=Xtallength, height=10, width=10, color=(1,0,1), opacity=0.2, up=norm(Xtalup))
XtalTop=box(pos=Xtal.up*Xtal.height/2, axis=Kdirection, length=Xtallength, height=0.01, width=10, color=(0,1,0), opacity=0.15, up=Xtal.up)
InBox=box(pos=-Xtal.length/2*Kdirection, axis=Kdirection, length=.01*Llength, height=15, width=15, color=(.5,.5,0), opacity=0.5, up=(0,1,0))
OutBox=box(pos=Xtal.length/2*Kdirection, axis=Kdirection, length=.01*Llength, height=15, width=15, color=(.5,.5,0), opacity=0.5, up=(0,1,0))

#----------------------------------------------------------------------------------------------
#EM Waves

# first segment: linear polarized wave
EInput1 = EMwave(StartPos=Startpositrue, PolDir=vector(0,1,0), Amplitude = 5,
                 K_Vector = Kdirection, length = (Llength-Xtal.length)/2, phase = 0, color=(1,0,0))
EInput1.CalcField(1,0)


# second segment: birefringent Xtal
EXtal1=EMwave(StartPos=EInput1.StartPos+EInput1.length*Kdirection, 
                        K_Vector=Kdirection, length=Xtal.length)
EXtal2=EMwave(StartPos=EXtal1.StartPos, K_Vector=Kdirection*(1+dn), length=Xtal.length)
EXtal3=EMwave(StartPos=EXtal1.StartPos, K_Vector=Kdirection, length=Xtal.length, color=(0,0,1))

# third segement: air again
EVac1=EMwave(StartPos=EXtal1.StartPos+EXtal1.length*Kdirection,
                                      K_Vector=Kdirection, length=(Llength-Xtal.length)/2, color=(1,0,0))
EVac2=EMwave(StartPos=EVac1.StartPos, K_Vector=Kdirection, length=(Llength-Xtal.length)/2, color=(1,0,0))
EVac3=EMwave(StartPos=EVac1.StartPos, K_Vector=Kdirection, length=(Llength-Xtal.length)/2, color=(1,1,0))

# update Xtal and Vac region after rotating the waveplate
def rotateXtal(AngleOfWP):
    phi=AngleOfWP
    Xtal.up=(0,cos(phi), sin(phi))
    XtalTop.pos=Xtal.up*Xtal.height/2
    XtalTop.up=Xtal.up
    
    # decompose incoming wave into Xtal-system
    PolDir1=Xtal.up
    PolDir2=cross(Xtal.up,Kdirection)
    A1=dot(PolDir1,EInput1.PolDir)*EInput1.Amplitude
    A2=dot(PolDir2,EInput1.PolDir)*EInput1.Amplitude
    EXtal1.PolDir=PolDir1
    EXtal1.Amplitude=A1
    EXtal1.phase=mag(EInput1.K_Vector*EInput1.length)
    EXtal1.CalcField(1,0)
    EXtal2.PolDir=PolDir2
    EXtal2.Amplitude=A2
    EXtal2.phase=mag(EInput1.K_Vector*EInput1.length)
    EXtal2.CalcField(1,0)

    # CoordSystem would be free, but we decompose in the incoming system
    PolDir=vector(0,1,0)
    A1=dot(PolDir,EXtal1.PolDir)*EXtal1.Amplitude
    A2=dot(PolDir,EXtal2.PolDir)*EXtal2.Amplitude
    PhaseVec=A1*exp( 1j*mag(EXtal1.K_Vector)*EXtal1.length-1j*EXtal1.phase )+A2*exp( 1j*mag(EXtal2.K_Vector)*EXtal1.length - 1j*EXtal2.phase )
    Phi1=-m.atan2(PhaseVec.imag,PhaseVec.real)
    EVac1.PolDir=PolDir
    EVac1.Amplitude=abs(PhaseVec)
    EVac1.phase=Phi1
    EVac1.CalcField(1,0)

    PolDir=vector(0,0,1)
    A1=dot(PolDir,EXtal1.PolDir)*EXtal1.Amplitude
    A2=dot(PolDir,EXtal2.PolDir)*EXtal2.Amplitude
    PhaseVec=A1*exp(1j*mag(EXtal1.K_Vector)*EXtal1.length-1j*EXtal2.phase)+A2*exp(1j*mag(EXtal2.K_Vector)*EXtal1.length-1j*EXtal2.phase)
    Phi2=-m.atan2(PhaseVec.imag,PhaseVec.real)
    EVac2.PolDir=PolDir
    EVac2.Amplitude=abs(PhaseVec)
    EVac2.phase=Phi2
    EVac2.CalcField(1,0)

#----------------------------------------------------------------------------------------------
## slider window
c = controls(x=0, y=0, width=100, height=800, range=60, title = "WP")
s1 = slider(pos=(0,-50), width=8, length=100, axis=(0,1,0), text = 'WPOrient (0..180)',   min=0, max=pi, action=lambda: rotateXtal(s1.value))
s1.value=AngleOfWP
text1 = button( pos=(0,55), width=13, height=12, text='Rot WP')

#----------------------------------------------------------------------------------------------



scene.fov=pi/10
scene.autoscale=1
scene.autoscale=0
t=0
dt=.05
while 1:
    rate(20)
    EInput1.CalcField(1,t)
    EXtal1.CalcField(1,t)
    EXtal2.CalcField(1,t)
    EVac1.CalcField(1,t)
    EVac2.CalcField(1,t)
    for i in range(EXtal1.nVec):
        EXtal3.Evecs[i].axis=EXtal1.Evecs[i].axis+EXtal2.Evecs[i].axis
 
    for i in range(EVac1.nVec):
        EVac3.Evecs[i].axis=EVac1.Evecs[i].axis+EVac2.Evecs[i].axis


    t=t+dt
    if scene.kb.keys:
        s= scene.kb.getkey()
        if s=='up':
            scene.center=scene.center-vector(0,1,0)
        elif s=='down':
            scene.center=scene.center+vector(0,1,0)
        elif s=='right':
            scene.center=scene.center-vector(1,0,0)
        elif s=='left':
            scene.center=scene.center+vector(1,0,0)

        else: s=scene.kb.getkey()
    c.interact()
